import paho.mqtt.client as mqtt
import threading
from loguru import logger


def mqttOnConnectCallback(client, userdata, flags, rc):
    userdata.onConnect()

def mqttOnMessageCallback(client, userdata, message):
    userdata.onMessage(message.topic, message.payload)

def mqttOnDisconnectCallback(client, userdata, rc):
    userdata.onDisconnect(rc)


class AbstractMqttPublisher(threading.Thread):
    def __init__(self, config, processImage):
        super().__init__()

        self.config = config["mqtt"]
        self.processImage = processImage

        self.client = mqtt.Client(userdata=self)

    def run(self):
        self.client.on_message = mqttOnMessageCallback
        self.client.on_connect = mqttOnConnectCallback
        self.client.on_disconnect = mqttOnDisconnectCallback

        if ("login" in self.config) and ("password" in self.config):
            self.client.username_pw_set(self.config["login"], self.config["password"])
        
        self.client.connect(self.config["broker"])
        self.client.loop_start()
        logger.info("mqtt loop started")

        self.localLoop()
    
    def localLoop(self):
        raise NotImplementedError()

    def onConnect(self):
        logger.info("mqtt connected")
    
    def onDisconnect(self, rc):
        logger.warning("mqtt disconnect, rc: {}".format(rc))

    def onMessage(self, topic, payload):
        logger.warning("mqtt unexpected message received: {} -> {}".format(topic, str(payload)))

