from pymodbus.client.sync import ModbusTcpClient as ModbusClient
from pymodbus.exceptions import ModbusIOException
from time import sleep
import threading
from loguru import logger


def modbusStart(config, processImage):
    modbusThread = threading.Thread(target=modbusHandler, args=[config, processImage])
    modbusThread.start()


def modbusHandler(config, processImage):
    modbusClient = config["modbus"]["client"]
    modbusRefreshPeriod = config["modbus"]["scanrate"]


    client = ModbusClient(modbusClient)

    try:
        client.connect()

        res = client.read_holding_registers(0x1010, 4)
        if isinstance(res, ModbusIOException):
            raise Exception(processImage)

        if len(res.registers) != 4:
            raise Exception("Unexpected number of registers for process image ({})".format(len(res.registers)))

        (analogOutputBits, analogInputBits, digitalOutputBits, digitalInputBits) = res.registers
        logger.debug(f"AO: {analogOutputBits}, AI: {analogInputBits}, DO: {digitalOutputBits}, DI: {digitalInputBits}")

        processImage.init(digitalOutputBits, digitalInputBits, analogInputBits)

        reg = client.read_coils(0, digitalOutputBits)
        if isinstance(reg, ModbusIOException):
            raise Exception(reg)
        with processImage:
            processImage.setCoils(reg.bits)

        while True:
            try:
                if not client.is_socket_open():
                    client.connect()

                reg = client.read_input_registers(0, analogInputBits // 8)
                if isinstance(reg, ModbusIOException):
                    raise Exception(reg)
                analogInputs = reg.registers

                reg = client.read_discrete_inputs(0, digitalInputBits)
                if isinstance(reg, ModbusIOException):
                    raise Exception(reg)
                discreteInputs = reg.bits

                coils = None
                with processImage:
                    processImage.setAnalogsInputs(analogInputs)
                    processImage.setDiscreteInputs(discreteInputs)
                    if processImage.hasPendingInputChanges():
                        processImage.notify()
                    if processImage.hasPendingOutputChanges():
                        coils = processImage.getCoils()

                if coils:
                    logger.debug("write coils to device: {}".format(coils))
                    reg = client.write_coils(0, coils)
                    if isinstance(reg, ModbusIOException):
                        raise Exception(reg)

            except Exception as e:
                logger.error("Exception in inner modbus handler loop: {}".format(e))
                client.close()
            finally:
                #sleep(modbusRefreshPeriod)
                if (processImage.coilEvent.wait(timeout=modbusRefreshPeriod)):
                    processImage.coilEvent.clear()


    except Exception as e:
        logger.error("Exception in modbus handler: {}".format(e))
    finally:
        client.close()
